from copy import copy

import matplotlib as mpl

from .plot_util import figsize


paint = {
    "figure.autolayout": False,  # Auto-apply `tight_layout`. (This doesn't seem to do
    #                              anything yet for the current simple plots).
    "axes.spines.top": False,
    "axes.spines.right": False,
    "figure.facecolor": "white",  # Avoid transparent background that make axis and tick
    #                               labels unreadable in a lot of image viewers (you
    #                               often see this when a matplotlib figure is eg shared
    #                               on Twitter).
    "axes.grid": True,
    "grid.color": "0.85",  # Very light gray
    "axes.axisbelow": True,  # Grid below patches (like histogram bars), not on top.
    "ytick.left": False,  # No ticks, but still labels
    "xtick.bottom": False,
    "xtick.color": "0.4",
    "ytick.color": "0.4",
    "xtick.major.pad": 0.5,
    "ytick.major.pad": 0.5,
}


default_figsize = figsize()

sizing = {
    "figure.figsize": default_figsize["figsize"],
    "figure.dpi": default_figsize["dpi"],
    "savefig.dpi": 2 * default_figsize["dpi"],  # to emulate "retina" option in nb's.
    "savefig.bbox": "tight",
    "xtick.labelsize": "small",  # Default is "medium"
    "ytick.labelsize": "small",
    "legend.fontsize": "small",  # Default is "medium"
}

new_style = {**paint, **sizing}

original_rcParams = copy(mpl.rcParams)
#   This is not the same as mpl.rcParamsDefault, as in a Jupyter Notebook, "backend" is
#   eg changed to "module://ipykernel.pylab.backend_inline" in mpl.rcParams.


def reset_and_apply():
    # Reset base style before applying new style, so we can interactively experiment
    # with changing the new style (without this reset, we can't experiment with
    # reverting previously set properties).
    mpl.rcParams.update(original_rcParams)
    mpl.rcParams.update(new_style)



"""
IPython notebook initialisation.
"""

# ━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━┓
# Give user feedback during slow imports.

from preload import preload

preload(["numpy", "matplotlib.pyplot", "pandas", "seaborn"])
# ────────────────────────────────────────────────────────────────────────────────────╯


#


# ━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━┓
import numpy as np
import matplotlib as mpl
import matplotlib.pyplot as plt
import seaborn as sns
import pandas as pd

print("Imported `np`, `mpl`, `plt`, `sns`, `pd`")
# ────────────────────────────────────────────────────────────────────────────────────╯


#


# ━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━┓
try:
    import IPython

    ipython = IPython.get_ipython()
    #   Will be `None` if running as a script (i.e. not in a notebook or IPython REPL).

except ImportError:
    
    ipython = False

if ipython:

    # ━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━┓
    # The following allows you to edit the source code of packages installed with `pip
    # install -e` (or `--editable)`, while running a notebook, and then use this updated
    # code in the notebook, without having to restart the kernel on every code change.
    # This works well but not perfectly, so kernel restarts may still be required.
    ipython.run_line_magic("reload_ext", "autoreload")
    ipython.run_line_magic("autoreload", "2")
    # Exclude ourself from autoreloading, to avoid an exponential recursive import
    # monster.
    ipython.run_line_magic("aimport", "-tfiers.nb")
    # ────────────────────────────────────────────────────────────────────────────────╯

    #

    # ━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━┓
    # If the last expression of a code cell is eg `product = 3 * 7`, and the cell is
    # run, IPython prints nothing, by default. Here, we make it print the result
    # (`21`). This avoids having to type an extra line with just `product` to see
    # the result. (Prints can still be suppressed by ending the line with `;`).

    from IPython.core.interactiveshell import InteractiveShell

    InteractiveShell.ast_node_interactivity = "last_expr_or_assign"
    # ────────────────────────────────────────────────────────────────────────────────╯

    #

    # ━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━┓
    # Double resolution plots (without taking up twice as much screen space).

    from IPython.display import set_matplotlib_formats

    set_matplotlib_formats("retina")
    # ────────────────────────────────────────────────────────────────────────────────╯

# ────────────────────────────────────────────────────────────────────────────────────╯


#


# ━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━┓
# Print precision for (floating point) numbers.


def set_print_precision(digits=4):
    # The ".4G" format allows eg `array([1000, 1E+05, 0.111])`
    number_format = f".{digits}G"
    # Print precision for plain Python numbers..
    if ipython:
        ipython.run_line_magic("precision", "%" + number_format)
    # ..and for NumPy.
    np.set_printoptions(
        formatter={
            "float_kind": lambda x: format(x, number_format),
        }
    )


set_print_precision()
# ────────────────────────────────────────────────────────────────────────────────────╯


#


# ━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━┓
# Misc imports

# Useful higher-order function from the Python standard library. Given a function,
# returns a copy of that function, but with some arguments already applied.
from functools import partial

# On import, augments Pandas dataframes with better wrangling API.
# import janitor

# Our own styling
from . import plot_style

plot_style.reset_and_apply()

# kwargs for creating new figures with a desired aspect ratio & width.
from .plot_util import figsize

# ────────────────────────────────────────────────────────────────────────────────────╯


#

# Reassignment trick to stop PyCharm from complaining about unused imports:
mpl, plt, sns, pd = mpl, plt, sns, pd
partial = partial
figsize = figsize



def figsize(
    aspect: float = 1.6,
    width: int = 500,
    rel_text_size: float = 1.33,
):
    """
    :param aspect:  Fig width / fig height.
    :param width:  Pixel width of figure.
    :param rel_text_size:  Determines how large text is relative to other plot elements,
                such as lines and ticks. (It is scaled so that it is "1" at the default
                dpi of 72).

    :return: A dictionary with keys "figsize" and "dpi" that can be passed as **kwargs
             to matplotlib figure creation methods.
    """
    dpi = 72 * rel_text_size  # = pixels per inch
    height = round(width / aspect)
    return dict(figsize=(width / dpi, height / dpi), dpi=dpi)



from setuptools import find_packages, setup


GITHUB_URL = "https://github.com/tfiers/tfiers-py"

with open("ReadMe.md", mode="r", encoding="utf-8") as f:
    readme = f.read()

setup(
    name="tfiers",
    description="My notebook setup and matplotlib style",
    author="Tomas Fiers",
    author_email="tomas.fiers@gmail.com",
    long_description=readme,
    long_description_content_type="text/markdown",
    url=GITHUB_URL,
    project_urls={"Source Code": GITHUB_URL},
    python_requires=">= 3.8",
    install_requires=[
        "preload ~= 2.2",
        "seaborn",
        # "pyjanitor",  # pip installing pyjanitor forces all optional extra deps of
        #                 janitor to be installed (like heavy pyspark etc). No thanks.
        #                 Conda does it right.
    ],
    packages=find_packages("src"),
    package_dir={"": "src"},  # (`""` is the "root" package).
    # Get package version from git tags
    setup_requires=["setuptools_scm"],
    use_scm_version={
        "version_scheme": "post-release",
        "local_scheme": "dirty-tag",
    },
)
